#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.permissions import ModifyPortalContent
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.Archetypes.public import Schema, registerType
from Products.Archetypes.public import StringField
from Products.Archetypes.public import StringWidget

from Products.StudyGuide.config import PROJECTNAME
from Products.StudyGuide.interfaces import IStudyGuide
from Products.StudyGuide.LabellessRadioColumn import LabellessRadioColumn
from Products.StudyGuide.ItemsColumn import ItemsColumn
from Products.StudyGuide.FixedWidthColumn import FixedWidthColumn
from Products.StudyGuide.getItems import getItems, getItemDicts

from Products.DataGridField import DataGridField, DataGridWidget

import sys, string

from config import *

from AccessControl import ClassSecurityInfo

security = ClassSecurityInfo()

StudyGuideSchema = ATContentTypeSchema.copy() + Schema((
    DataGridField('items',            
                  columns = ('knowledge_level', 'item', 'notes'),
                  default = getItemDicts(),
                  allow_delete = False,
                  allow_insert = False,
                  allow_reorder = False,
                  allow_empty_rows = False,
                  widget = DataGridWidget(label="Items",
                                          description="For each item, set your knowledge level Level increases toward the right; level 5 is best. If you set your knowledge level for a heading, you have to manually click knowledge level on subheadings or items within that heading. If you want to add your own items, click any of the plus signs to add a row above that position. You can add a note to yourself in the right column.",
                                          columns= {
                                                   "knowledge_level" : LabellessRadioColumn("Knowledge", vocabulary="getKnowledgeLevels", default="One"),
                                                   "item" : ItemsColumn("Item", getItems()),
                                                   "notes" : FixedWidthColumn("Notes"),
                                                   }
                                          ),
                    ),
    ))

# Finalise the schema according to ATContentTypes standards. This basically
# moves the Related items and Allow discussion fields to the bottom of the
# form. See ATContentTypes.content.schemata for details.
finalizeATCTSchema(StudyGuideSchema)

class StudyGuide(ATCTContent):
    """
    A StudyGuide.
    """
    # Standard content type setup
    portal_type = meta_type = 'StudyGuide'
    content_icon = 'studyguideicon.png'
    schema = StudyGuideSchema
    typeDescription= 'A study guide document'
    typeDescMsgId  = 'StudyGuide_description_edit'
    archetype_name = 'StudyGuide'
    #immediate_view = 'StudyGuide_view'
    #default_view   = 'StudyGuide_view'

    actions = (#{'id': 'view',
               # 'name': 'View',
               # 'action': 'string:${object_url}/StudyGuide_view',
               # 'permissions': (CMFCorePermissions.View,)},
               {'id': 'references',
                'name': 'References',
                'action': 'string:${object_url}/reference_edit',
                'permissions': (CMFCorePermissions.ModifyPortalContent,),
                'visible':0},
               {'id': 'metadata',
                'name': 'Properties',
                'action': 'string:${object_url}/base_metadata',
                'permissions': (CMFCorePermissions.ModifyPortalContent,),
                'visible':0})

    # title-to-id generation when an object is created
    _at_rename_after_creation = True

    def getKnowledgeLevels(self):
        return DisplayList((("one", "One",),
                            ("two", "Two",),
                            ("three", "Three",),
                            ("four", "Four",),
                            ("five", "Five",),
                          ))

registerType(StudyGuide, PROJECTNAME)
