#!/usr/bin/python
# -*- coding: utf-8 -*-

# This script was modified from generator.py in DIYPloneStyle.

import os, sys, optparse

# command line parser
parser = optparse.OptionParser()

parser.add_option('-p','--productname',
    action='store',
    dest='productname',
    help="Name that will be given to the product")

parser.add_option('-a','--authorstring',
    action='store',
    dest='authorstring',
    help="Author text string, e.g. Jeff Pittman <geojeff@geojeff.org>")

parser.add_option('-s','--stxfile',
    action='store',
    dest='stxfile',
    help="Text file containing structured text items for the study guide.")

parser.add_option('-v', '--verbose',
    action="store_true",
    dest="verbose",
    help="Verbose mode")

parser.add_option('-r', '--replace',
    action="store_true",
    dest="replace",
    help="Replace original")

options, args = parser.parse_args()

path = os.path.dirname( os.path.abspath(__file__) )
if path.endswith('StudyGuide%sbin' % os.sep):
    path = path[:-3]
else:
    raise IOError, 'This script must be in the StudyGuide%sbin/ directory.' % os.sep

# Output help if no argument is given while calling the script
if not options.productname and len(args) != 1 or not os.path.isdir(path):
    parser.parse_args(args=['-h'])

PRODUCT_NAME = options.productname or args[0]

# check for valid product name
if PRODUCT_NAME[0] in '0123456789':
    raise IOError, "Product name cannot start with a digit"
else:
    pass

REPLACE_ORIGINAL = options.replace

PRODUCT_AUTHOR = options.authorstring or 'John Doe <john.doe@dev.null>'

STX_FILE = options.stxfile

GET_ITEMS_TEXT_HEADER = """
from Products.PythonScripts.standard import structured_text

# See usage notes at bottom

items_text = \"\"\""""

GET_ITEMS_TEXT_FOOTER = '\"\"\"'

DEFAULT_GET_ITEMS_TEXT = """
Main Heading

  Subheading 1

    * bullet one

    * bullet two

    * bullet three

  Subheading 2

    word1 -- definition of word1

    word2 -- definition of word2

  **Subheading 3** (emphasized)

    Text under subheading 3, showing stx formatting such as *italics*, _underlining*, and 'inline preformatted text'.

Main Heading 2

  Subheading 4, Showing a Table

    |---------------------------------------------------------------|<newline>| Tool                       | Use                              |<newline>|===============================================================|<newline>| Screwdriver                | Used for screwing screws.        |<newline>|---------------------------------------------------------------|<newline>| Band Saw                   | Used for cutting metal and wood. |<newline>|---------------------------------------------------------------|<newline>| Table Saw                  | Used for sawing wood.            |<newline>|---------------------------------------------------------------|<newline>| Lathe                      | Used for shaping wood.           |<newline>|---------------------------------------------------------------|<newline>| Sander                     | Used for sanding wood.           |<newline>|---------------------------------------------------------------|

  Subheading 5, showing preformatted text

    The following is preformatted text::<newline><newline>      Screwdriver     Used for screwing screws.<newline>      Band Saw        Used for cutting metal and wood.<newline>      Table Saw       Used for sawing wood.<newline>      Lathe           Used for shaping wood.<newline>      Sander          Used for sanding wood.

  Subheading 6, showing links

    "www.zope.org":http://www.zope.org

    "www.plone.org":http://www.plone.org

    "www.python.org":http://www.python.org

  Subheading 7, showing images

    This is the image used for knowledge level 5: "Knowledge Level 5":img:level-five.png

    Does this show?
"""

GET_ITEMS_CODE = """

# Use this utility to generate a default list of row data from a simple indented
# outline in a text file called items.in.  Use blanks to indent, and indent one
# space for every 10 pixels for the indentation.  If you would like to change
# this, change pixel_to_space_factor.
#
# After generating items.out, go into StudyGuide.py and insert it in the
# parameter setting part of DataGridField('outline_data').  Then, install and test.
#

pixel_to_space_factor = 10

class Item:
    def __init__(self, text):
        text_lstripped = text.lstrip()
        self.indentation = (len(text) - len(text_lstripped)) * pixel_to_space_factor
        if len(text_lstripped.strip()) == 0:
            self.text = ''
        else:
            text = text.replace('<newline>', '\\n')
            stx = structured_text(text)
            self.text = stx.replace('<apostrophe>', "'")

    def __str__(self):
        return '[%s, %s]' % (self.indentation, self.text)

    def __repr__(self):
        return '%s(%s, %s)' % (
            self.__class__.__name__,  # the instance class name
            self.indentation, self.text
        )

    def getDict(self):
        return {'knowledge_level' : 'one', 'item' : "<div style='margin: 0px 0px 0px %dpx'>%s</div>" % (self.indentation, self.text), 'notes' : ''}

def loadItems():
    items = []
    for line in items_text.split('\\n'):
        line = line.rstrip()
        if len(line.strip()) > 0:
            items.append(Item(line))
    return items

def readItems():
    items_in = open("items.in", "r")
    items = []
    for line in items_in.readlines():
        line = line.rstrip()
        if len(line.strip()) > 0:
            items.append(Item(line))
    items_in.close()
    return items

def getItems():
    return loadItems()

def getItemDicts():
    items = loadItems()
    # Why the -1 needed here? Something askew.
    return [{'knowledge_level' : 'one', 'item' : '', 'notes' : ''} for x in range(len(items))]

# It turns out that setting the default for the DataGridField 
# is necessary as a kind of placeholder, it seems, but there
# needn't be the actual text items in the initial load, as the
# empty edit view doesn't happen.  Not sure, but getItemDicts()
# above is doing the job, and the one below is unnecessary.
def getItemDicts2():
    items = loadItems()
    items_dict_list = []
    items_dict_list.append(items[0].getDict())
    for item in items[1:-2]:
        items_dict_list.append(item.getDict())
    items_dict_list.append(items[-1].getDict())
    return items_dict_list
"""

if STX_FILE:
    GET_ITEMS_TEXT = open(STX_FILE, 'r').read()
else:
    GET_ITEMS_TEXT = DEFAULT_GET_ITEMS_TEXT

TEXT_FILES = {
    'HISTORY.txt': '''Changelog for %s

    (name of developer listed in brackets)

%s 1.0 - Unreleased

    - Generated skeleton product from StudyGuide
      via use of the generator.py script.
''' % ( PRODUCT_NAME, PRODUCT_NAME ),

    'README.txt': '''Description

    %s is a product that presents a list of items to a user
    with a set of radio buttons alongside each item for 
    rating knowledge level.  There is also a notes field for
    each item.  The list of items is loaded from a python
    script called getItems.py.  The list of items may
    be entered manually within getItems.py or it may
    be provided to the createStudyGuide.py script as a text
    file.  Either way, the list of items is a structured
    text block.  Each line (separated by blank lines) in
    the block of text will become an individual item in the
    list of items presented to the user. This is best learned
    by looking at the default example in getItems.py.

    %s is based on StudyGuide, a skeleton product ready for 
    building new StudyGuide variant instances for Plone.

Installation

    First, %s requires DataGridField. Install it as a separate
    product in the Products directory of your Zope Instance.

    Then, place %s in the Products directory of your Zope instance
    and restart the server.

    Go to the 'Site Setup' page in the Plone interface and click on the
    'Add/Remove Products' link.

    Choose the product (check its checkbox) and click the 'Install' button.

    Uninstall -- This can be done from the same management screen, but only
    if you installed it from the quick installer.

    Note: You may have to empty your browser cache to see the effects of the
    product installation.

Written by

    %s
''' % ( PRODUCT_NAME, PRODUCT_NAME, PRODUCT_NAME, PRODUCT_NAME, PRODUCT_AUTHOR ),
    'version.txt': '1.0',
    'getItems.py' : '%s%s%s%s' % (GET_ITEMS_TEXT_HEADER, GET_ITEMS_TEXT, GET_ITEMS_TEXT_FOOTER, GET_ITEMS_CODE),
   }

UNWANTED_FOLDERS_AND_FILES = (
    'bin',
    )

UNWANTED_FILE_EXTENSIONS = ('.pyc', '.pyo', '.swp', )
UNWANTED_FILE_PREFIXES = ('.', '#', )

FILE_MAPPING = (
    ('StudyGuide.py', '%s.py' % PRODUCT_NAME),
    ('StudyGuideInterface.py', '%sInterface.py' % PRODUCT_NAME),
    ('StudyGuide_view.pt', '%s_view.pt' % PRODUCT_NAME),
    ('StudyGuide.css.dtml', '%s.css.dtml' % PRODUCT_NAME),
    ('StudyGuide', PRODUCT_NAME),
    ('studyguideicon.png', '%sicon.png' % PRODUCT_NAME.lower()),
    )

CODE_EXTENSIONS = ('.py', '.pt', '.dtml', '.props', '.xml')

CODE_MAPPING = (
    ('StudyGuide', PRODUCT_NAME),
    ('studyguide', PRODUCT_NAME.lower()),
    ('studyguideicon', '%sicon' % PRODUCT_NAME.lower()),
   )

if PRODUCT_AUTHOR:
    CODE_MAPPING = CODE_MAPPING + (
        ('Jeff Pittman <geojeff@geojeff.org>', PRODUCT_AUTHOR),
       )

def log(message):
    if options.verbose:
        print >> sys.stdout, message

# Remove directory
def rmdir(dirname):
    log('Remove directory %s' % dirname)
    if os.path.isdir( dirname ):
        for filename in os.listdir(dirname):
            filepath = os.path.join(dirname, filename)
            if os.path.isfile(filepath):
                os.remove(os.path.join(dirname, filename))
            else:
                rmdir(filepath)
        os.rmdir(dirname)

# Test if the item must be kept in the new product.
def isItemKept(item):
    if item in UNWANTED_FOLDERS_AND_FILES: return False
    for unwanted in UNWANTED_FILE_EXTENSIONS:
        if item.endswith(unwanted): return False
    for unwanted in UNWANTED_FILE_PREFIXES:
        if item.startswith(unwanted): return False
    return True

# Recursive copy of a folder content 
def generateDirectoryCopy(srcDirectory, dstDirectory):
    if not os.path.exists(dstDirectory):
        os.mkdir(dstDirectory)
        log('Created directory %s' % dstDirectory)
    for item in os.listdir(srcDirectory):
        if isItemKept(item):
            # Get the name of the copied item in the new product.
            dst_item = item
            for src, dst in FILE_MAPPING:
                if dst_item.endswith(src):
                    dst_item = dst_item.strip(src) + dst
            # Get source and dest paths
            src_path = os.path.join(srcDirectory, item)
            dst_path = os.path.join(dstDirectory, dst_item)
            # Read, modify and write to destination if item is a file
            if os.path.isfile(src_path):
                # Read source file
                f_src = open(src_path,'r')
                data = f_src.read()
                f_src.close()
                # Full text replacement
                if item in TEXT_FILES.keys():
                    data = TEXT_FILES[item]
                # Code subsitutions
                f,ext = os.path.splitext(item)
                if ext in CODE_EXTENSIONS or item in ('LICENSE.txt',):
                    # Replace PRODUCT_NAME, SKINNAME, etc.
                    for orig, dest in CODE_MAPPING:
                        data = data.replace(orig, dest)
                # Write destination file
                f_dst = open(dst_path,'w')
                f_dst.write(data)
                f_dst.close()
                log('Created file %s' % dst_path)

            # Create and copy content if item is a folder
            elif os.path.isdir(src_path):
                if not os.path.exists(dst_path):
                    os.mkdir(dst_path)
                    log('Created directory %s' % dst_path)
                generateDirectoryCopy(src_path, dst_path)

generateDirectoryCopy( path, path.replace('StudyGuide', PRODUCT_NAME) )
log('Created %s based on StudyGuide.' % PRODUCT_NAME)

if REPLACE_ORIGINAL:
    rmdir( os.path.join(path.strip('StudyGuide'), 'StudyGuide') )
    log('Removed StudyGuide.')

log('%s product generated successfully.' % PRODUCT_NAME)
